{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSSlider;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  extctrls;

const FBoundMin = 0;

type
  TSliderDirection = (sdUp, sdDown);
  TSliderAlignment = (saLeft, saRight);
  TSliderShape = (ssHalfRound, ssRectangle, ssRounded);
  TTimerMouse = record
                 mbtn : TMouseButton;
                 shst : TShiftState;
                 xmouse : Integer;
                 ymouse : Integer;
                end;
  TLCARSSlider = class(TGraphicControl)
  private
   FMinValue: Integer;
   FMaxValue: Integer;
   FCurValue: Integer;
   FTextOnSlider: Boolean;
   FMarginWidth : integer;
   FSliderWidth : integer;
   FSliderColor : TColor;
   FMarginColor : TColor;
   FSliderAlignment : TSliderAlignment;
   FBackColor: TColor;
   FSliderThickness : Integer;
   FSliderShape : TSliderShape;
   FSliderDirection : TSliderDirection;
   FSliderYLow : Integer;
   FSliderYHigh : Integer;
   FMarginActive : Boolean;
   FPercentPlace : Integer;
   FZeroPercent : Integer;
   FHundredPercent : Integer;
   FMarginRadius : Integer;
   FPositionPlace : Integer;
   FProgressToShow : Integer;
   FOldToShow : Integer;
   FTextWidth : Integer;
   FTextHeight : Integer;
   FTextPosX : Integer;
   FTextPosY : Integer;
   FSliderGrabbed : Boolean;
   FOldMarkerPos : Integer;
   FMouseIsPressed : Boolean;
   FTimerMouse : TTimerMouse;
   FBack : TBitmap;
   FSliderTimer : TTimer;
   FOnChange : TNotifyEvent;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   procedure SetTextOnSlider(Value: Boolean);
   procedure SetMinValue(Value: Integer);
   procedure SetMaxValue(Value: Integer);
   procedure SetPosition(Value: Integer);
   procedure SetSliderColor(Value : TColor);
   procedure SetMarginColor(Value : TColor);
   procedure SetSliderWidth(Value : Integer);
   procedure SetMarginRadius(Value : Integer);
   procedure SetSliderThickness(Value : Integer);
   procedure SetSliderAlignment(Value : TSliderAlignment);
   procedure SetSliderShape(Value : TSliderShape);
   procedure SetSliderDirection(Value : TSliderDirection);
   procedure SetMarginActive(Value : Boolean);
   function CalculateNewPosition : Boolean;
   function OverSlider(x, y : Integer) : Boolean;
   procedure MoveSlider(x, y : Integer);
   procedure UpdatePosition(x, y : Integer);
   procedure SliderTimerTimer(Sender: TObject);
   function ValidMouseArea(xdir : Integer) : Boolean;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
   procedure Paint; override;
   procedure Change ; dynamic;
   procedure MouseEnter; dynamic;
   procedure MouseLeave; dynamic;
   procedure MouseDown(Button:TMouseButton;
             Shift:TShiftState;X,Y:Integer);override;
   procedure MouseMove(Shift:TShiftState;
             X,Y:Integer);override;
   procedure MouseUp(Button:TMouseButton;
             Shift:TShiftState;X,Y:Integer);override;
  public
   property MouseInControl: Boolean read FMouseInControl;
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure AddPosition(Value: Integer);
  published
   property Align;
   property Anchors;
   property Constraints;
   property Enabled;
   property Font;
   property SliderAlignment : TSliderAlignment read FSliderAlignment
    write SetSliderAlignment;
   property SliderShape : TSliderShape read FSliderShape
    write SetSliderShape;
   property SliderColor : TColor read FSliderColor write SetSliderColor;
   property MarginColor : TColor read FMarginColor write SetMarginColor;
   property SliderWidth : Integer read FSliderWidth write SetSliderWidth;
   property SliderDirection : TSliderDirection read FSliderDirection
    write SetSliderDirection;
   property MarginRadius : Integer read FMarginRadius write SetMarginRadius;
   property MinValue: Integer read FMinValue write SetMinValue default 0;
   property MaxValue: Integer read FMaxValue write SetMaxValue default 100;
   property MarginActive : Boolean read FMarginActive write SetMarginActive;
   property ParentColor;
   property ParentFont default true;
   property ParentShowHint;
   property Position: Integer read FCurValue write SetPosition default 0;
   property ShowHint;
   property TextOnSlider: Boolean read FTextOnSlider write SetTextOnSlider default True;
   property Visible;
   property SliderThickness : integer read FSliderThickness
    write SetSliderThickness;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnChange: TNotifyEvent read FOnChange write FOnChange;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSSlider.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FPositionPlace := 1;
 Width := 50;
 Height := 200;
 SliderWidth := Width div 2;
 FSliderThickness := 17;
 FSliderAlignment := saLeft;
 FSliderDirection := sdUp;
 FMinValue := 0;
 FMaxValue := 100;
 Position := 0;
 FTextOnSlider := True;
 FSliderColor := clyellow;
 FMarginColor := clred;
 FMarginActive := true;
 ControlStyle := ControlStyle + [csOpaque];
 {Canvas.Brush.Style := bsClear;}
 FBack := TBitmap.Create;

end;

destructor TLCARSSlider.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSSlider.Paint;
var VisibleRect, BackRect : TRect;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 if (FSliderWidth + FMarginWidth) <> Width then
  begin
   FSliderWidth := FSliderWidth + (Width - FSliderWidth - FMarginWidth);
   FMarginWidth := Width - FSliderWidth;
  end;
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0,0, FBack.Width, FBack.Height);
 FBack.Canvas.Brush.Color := FMarginColor;
 FBack.Canvas.Pen.Color := FMarginColor;
 FBack.Canvas.RoundRect(0, 0, FBack.Width, FBack.Height,
                        MarginRadius, MarginRadius);
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 if FSliderAlignment = saRight then
  FBack.Canvas.Rectangle(FMarginWidth - 2, 0, FBack.Width, FBack.Height)
 else if FSliderAlignment = saLeft then
  FBack.Canvas.Rectangle(0, 0 , FBack.Width - FMarginWidth + 2, FBack.Height);
 FTextWidth := FBack.Canvas.TextWidth(InttoStr(Position));
 FTextHeight := FBack.Canvas.TextHeight(InttoStr(Position));
 FPositionPlace := MaxValue - MinValue;
 FZeroPercent := FSliderThickness div 2;
 FHundredPercent := FBack.Height - (FSliderThickness div 2);
 FPercentPlace := FHundredPercent - FZeroPercent;
 CalculateNewPosition;
 FBack.Canvas.Brush.Color := FSliderColor;
 FBack.Canvas.Pen.Color := FSliderColor;
 if FSliderDirection = sdUp then
  begin
   FTextPosY := FBack.Height - FZeroPercent - FProgressToShow - (FTextHeight div 2);
   FSliderYLow := FBack.Height - FZeroPercent - FProgressToShow - (FSliderThickness div 2);
   FSliderYHigh := FBack.Height - FZeroPercent - FProgressToShow + (FSliderThickness div 2);
  end
 else if FSliderDirection = sdDown then
  begin
   FTextPosY := FZeroPercent + FProgressToShow - (FTextHeight div 2);
   FSliderYLow := FzeroPercent + FProgressToShow - (FSliderThickness div 2);
   FSliderYHigh := FzeroPercent + FProgressToShow + (FSliderThickness div 2);
  end;
 if FSliderAlignment = saLeft then
  begin
   if FSliderShape = ssHalfRound then
    begin
     FBack.Canvas.Ellipse(0, FSliderYLow, FSliderThickness, FSliderYHigh);
     FBack.Canvas.Rectangle(FSliderThickness div 2, FSliderYLow,
                            FSliderWidth - 2, FSliderYHigh);
    end
   else if FSliderShape = ssRectangle then
    begin
     FBack.Canvas.Rectangle(0, FSliderYLow, FSliderWidth - 2, FSliderYHigh);
    end
   else if FSliderShape = ssRounded then
    begin
     FBack.Canvas.RoundRect(0, FSliderYLow, FSliderWidth - 2, FSliderYHigh,
                            FSliderThickness, FSliderThickness);
    end;
   if FTextOnSlider then
    FTextPosX := ((FSliderWidth - 2) div 2) - (FTextWidth div 2)
   else
    begin
     FBack.Canvas.Brush.Color := FMarginColor;
     FBack.Canvas.Pen.Color := FMarginColor;
     FTextPosX := FBack.Width - ((FMarginWidth - 2) div 2) - (FTextWidth div 2);
    end;
  end
 else if FSliderAlignment = saRight then
  begin
   if FSliderShape = ssHalfRound then
    begin
     FBack.Canvas.Ellipse(FBack.Width - FSliderThickness + 1, FSliderYLow,
                          FBack.Width, FSliderYHigh);
     FBack.Canvas.Rectangle(FBack.Width - FSliderWidth + 3, FSliderYLow,
                            FBack.Width - (FSliderThickness div 2) + 1, FSliderYHigh);
    end
   else if FSliderShape = ssRectangle then
    begin
     FBack.Canvas.Rectangle(FBack.Width - FSliderWidth + 3, FSliderYLow,
                            FBack.Width, FSliderYHigh);
    end
   else if FSliderShape = ssRounded then
    begin
     FBack.Canvas.RoundRect(FBack.Width - FSliderWidth + 3, FSliderYLow,
                            FBack.Width, FSliderYHigh,
                            FSliderThickness, FSliderThickness);
    end;
   if FTextOnSlider then
    FTextPosX := FBack.Width - ((FSliderWidth - 2) div 2) - (FTextWidth div 2)
   else
    begin
     FBack.Canvas.Brush.Color := FMarginColor;
     FBack.Canvas.Pen.Color := FMarginColor;
     FTextPosX := ((FMarginWidth - 2) div 2) - (FTextWidth div 2);
    end;
  end;
 FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Position));
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, FBack.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSSlider.UpdatePosition(x, y : Integer);
var Factor : real;
begin
 Factor := FPercentPlace / FPositionPlace;
 if FSliderDirection = sdUp then
  FProgressToShow := FBack.Height - FZeroPercent - y
 else if FSliderDirection = sdDown then
  FProgressToShow := y - FZeroPercent;
 if FProgressToShow < 0 then FProgressToShow := 0;
 if FProgressToShow > FHundredPercent then FProgressToShow := FHundredPercent;
 Position := Round(FProgressToShow / Factor) + FMinValue;
end;

function TLCARSSlider.CalculateNewPosition : boolean;
var Factor : real;
begin
 FOldToShow := FProgressToShow;
 Factor := FPercentPlace / FPositionPlace;
 FProgressToShow := Round(Factor * (FCurValue - FMinValue));
 if not FSliderGrabbed then
  begin
   if FSliderDirection = sdUp then
    FOldMarkerPos := FBack.Height - FProgressToShow
   else if FSliderDirection = sdDown then
    FOldMarkerPos := FProgressToShow;
  end;
 if FProgressToShow = FOldToShow then
  CalculateNewPosition := false
 else
  CalculateNewPosition := true;
end;

function TLCARSSlider.OverSlider(x, y : Integer) : Boolean;
begin
 OverSlider := false;
 if FSliderDirection = sdUp then
  begin
   if (y >= FBack.Height - FZeroPercent - FProgressToShow - (SliderThickness div 2)) and
      (y <= FBack.Height - FZeroPercent - FProgressToShow + (SliderThickness div 2)) and
      ValidMouseArea(X) then
    OverSlider := true;
  end
 else if FSliderDirection = sdDown then
  begin
   if (y >= FZeroPercent + FProgressToShow - (SliderThickness div 2)) and
      (y <= FZeroPercent + FProgressToShow + (SliderThickness div 2)) and
      ValidMouseArea(X) then
    OverSlider := true;
  end
end;

procedure TLCARSSlider.MoveSlider(x, y : Integer);
begin
 if FSliderDirection = sdUp then
  begin
   if y > FOldMarkerPos then
    begin
     AddPosition(-1);
    end
   else if y < FOldMarkerPos then
    begin
     AddPosition(1);
    end;
  end
 else if FSliderDirection = sdDown then
  begin
   if y > FOldMarkerPos then
    begin
     AddPosition(1);
    end
   else if y < FOldMarkerPos then
    begin
     AddPosition(-1);
    end;
  end;
end;

function TLCARSSlider.ValidMouseArea(xdir : Integer) : Boolean;
begin
 ValidMouseArea := true;
 if MarginActive = false then
  begin
   if ((FSliderAlignment = saLeft) and (xdir > (FSliderWidth - 2))) or
      ((FSliderAlignment = saRight) and (xdir < (FMarginWidth + 2))) then
    ValidMouseArea := false;
  end;
end;

procedure TLCARSSlider.MouseDown(Button:TMouseButton;
       Shift:TShiftState;X,Y:Integer);
begin
 inherited MouseDown(Button,Shift,X,Y);
 if ValidMouseArea(X) then
 begin
  FTimerMouse.mbtn := Button;
  FTimerMouse.shst := Shift;
  FTimerMouse.xmouse := x;
  FTimerMouse.ymouse := y;
  FMouseIsPressed := true;
  if (Button = mbLeft) and (OverSlider(x, y)) then
   FSliderGrabbed := true;
  if (Button = mbLeft) and not OverSlider(x, y) and
     (FMouseIsPressed = true) then
   begin
    if FSliderTimer = nil then
     begin
      FSliderTimer := TTimer.Create(self);
      FSliderTimer.OnTimer := SliderTimerTimer;
      FSliderTimer.Interval := 500;
     end
    else
     begin
      FSliderTimer.OnTimer := SliderTimerTimer;
      FSliderTimer.Interval := 100;
     end;
    FSliderTimer.Enabled := true;
    MoveSlider(x, y);
   end;
 end;
end;

Procedure TLCARSSlider.MouseUp(Button:TMouseButton;
       Shift:TShiftState;X,Y:Integer);
begin
 FMouseIsPressed := false;
 if (Button = mbLeft) and (FSliderGrabbed) then
  FSliderGrabbed := False;
 if FSliderTimer <> nil then
  begin
   FSliderTimer.Enabled := false;
   FSliderTimer := nil;
  end;
 inherited MouseUp(Button,Shift,X,Y);
end;

procedure TLCARSSlider.MouseMove(Shift:TShiftState;X,Y:Integer);
begin
 FTimerMouse.xmouse := x;
 FTimerMouse.ymouse := y;
 if FSliderGrabbed then
  begin
   UpdatePosition(x, y);
   FOldMarkerPos := y;
  end;
 if (ssLeft in Shift) and (OverSlider(x, y)) then
  begin
   FSliderGrabbed := true;
  end
 else FSliderGrabbed := false;
end;

procedure TLCARSSlider.SliderTimerTimer(Sender: TObject);
begin
 if FMouseIsPressed and MouseCapture then
  begin
    try
     MouseDown(FTimerMouse.mbtn, FTimerMouse.shst,
               FTimerMouse.xmouse, FTimerMouse.ymouse);
    except
     FSliderTimer.Enabled := False;
     raise;
    end;
  end;
end;

procedure TLCARSSlider.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TLCARSSlider.SetTextOnSlider(Value: Boolean);
begin
  if Value <> FTextOnSlider then
  begin
    FTextOnSlider := Value;
    Repaint;
  end;
end;

procedure TLCARSSlider.SetSliderColor(Value: TColor);
begin
  if Value <> FSliderColor then
  begin
    FSliderColor := Value;
    Repaint;
  end;
end;

procedure TLCARSSlider.SetMarginColor(Value: TColor);
begin
  if Value <> FMarginColor then
  begin
    FMarginColor := Value;
    Repaint;
  end;
end;

procedure TLCARSSlider.SetSliderWidth(Value: integer);
begin
  if Value <> FSliderWidth then
  begin
    FSliderWidth := Value;
    FMarginWidth := Width - Value;
    Repaint;
  end;
end;

procedure TLCARSSlider.SetMarginRadius(Value: integer);
begin
  if Value <> FMarginRadius then
  begin
    FMarginRadius := Value;
    Repaint;
  end;
end;

procedure TLCARSSlider.SetSliderAlignment(Value: TSliderAlignment);
begin
 if Value <> FSliderAlignment then
  begin
   FSliderAlignment := Value;
   Repaint;
  end;
end;

procedure TLCARSSlider.SetSliderThickness(Value: Integer);
begin
 if Value <> FSliderThickness then
  begin
   FSliderThickness := Value;
   Repaint;
  end;
end;

procedure TLCARSSlider.SetSliderShape(Value: TSliderShape);
begin
 if Value <> FSliderShape then
  begin
   FSliderShape := Value;
   Repaint;
  end;
end;

procedure TLCARSSlider.SetSliderDirection(Value: TSliderDirection);
begin
 if Value <> FSliderDirection then
  begin
   FSliderDirection := Value;
   Repaint;
  end;
end;

procedure TLCARSSlider.SetMarginActive(Value: Boolean);
begin
 if Value <> FMarginActive then
  begin
   FMarginActive := Value;
   Repaint;
  end;
end;

procedure TLCARSSlider.SetMinValue(Value: Integer);
begin
  if Value <> FMinValue then
  begin
    if Value > FMaxValue then
     FMinValue := FCurValue
    else
     FMinValue := Value;
    if FCurValue < Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TLCARSSlider.SetMaxValue(Value: Integer);
begin
  if Value <> FMaxValue then
  begin
    if Value < FMinValue then
     FMaxValue := FCurValue
    else
     FMaxValue := Value;
    if FCurValue > Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TLCARSSlider.SetPosition(Value: Integer);
begin
 if Value < FMinValue then
  Value := FMinValue
 else if Value > FMaxValue then
  Value := FMaxValue;
 if FCurValue <> Value then
  begin
   FCurValue := Value;
   Change;
  end;
 Repaint;
end;

procedure TLCARSSlider.AddPosition(Value: Integer);
begin
  Position := FCurValue + Value;
end;

procedure TLCARSSlider.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSSlider.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSSlider.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSSlider.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSSlider]);
end;

end.
